
/* Chrysalide - Outil d'analyse de fichiers binaires
 * symbol.h - prototypes pour la gestion des symboles dans un binaire
 *
 * Copyright (C) 2009-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _FORMAT_SYMBOL_H
#define _FORMAT_SYMBOL_H


#include <glib-object.h>


#include "../analysis/routine.h"
#include "../analysis/db/item.h"
#include "../analysis/db/items/comment.h"
#include "../arch/instruction.h"
#include "../glibext/linegen.h"



/* --------------------- FONCTIONNALITES BASIQUES POUR SYMBOLES --------------------- */


/* Types de symbole */
typedef enum _SymbolType
{
    STP_DATA,                               /* Données brutes              */
    STP_ROUTINE,                            /* Simple morceau de code      */
    STP_CODE_LABEL,                         /* Renvoi au sein de code      */
    STP_OBJECT,                             /* Objet quelconque            */
    STP_FUNCTION,                           /* Simple morceau de code      */
    STP_ENTRY_POINT,                        /* Morceau de code en entrée   */
    STP_STRING,                             /* Chaîne de caractères        */
    STP_RO_STRING,                          /* Chaîne de caractères        */

    STP_COUNT

} SymbolType;


#define HAS_DATA_INSTR(type) (type == STP_DATA || type == STP_RO_STRING)



#define G_TYPE_BIN_SYMBOL               g_binary_symbol_get_type()
#define G_BIN_SYMBOL(obj)               (G_TYPE_CHECK_INSTANCE_CAST((obj), g_binary_symbol_get_type(), GBinSymbol))
#define G_IS_BIN_SYMBOL(obj)            (G_TYPE_CHECK_INSTANCE_TYPE((obj), g_binary_symbol_get_type()))
#define G_BIN_SYMBOL_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_BIN_SYMBOL, GBinSymbolClass))
#define G_IS_BIN_SYMBOL_CLASS(klass)    (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_BIN_SYMBOL))
#define G_BIN_SYMBOL_GET_CLASS(obj)     (G_TYPE_INSTANCE_GET_CLASS((obj), G_TYPE_BIN_SYMBOL, GBinSymbolClass))


/* Symbole d'exécutable (instance) */
typedef struct _GBinSymbol GBinSymbol;

/* Symbole d'exécutable (classe) */
typedef struct _GBinSymbolClass GBinSymbolClass;


/* Indique le type défini pour un symbole d'exécutable. */
GType g_binary_symbol_get_type(void);

/* Crée un nouveau symbole d'exécutable. */
GBinSymbol *g_binary_symbol_new(SymbolType);

/* Compare deux symboles d'exécutable selon leurs propriétés. */
int g_binary_symbol_cmp(const GBinSymbol **, const GBinSymbol **);

/* Compare un symbole et une localisation. */
int g_binary_symbol_cmp_with_vmpa(const GBinSymbol *, const vmpa2t *);

/* Fournit le type du symbole. */
SymbolType g_binary_symbol_get_target_type(const GBinSymbol *);

/* Définit si un symbole est susceptible de démarrer un bloc. */
void g_binary_symbol_define_as_block_start(GBinSymbol *, bool);

/* Indique si un symbole est susceptible de démarrer un bloc. */
bool g_binary_symbol_is_block_start(GBinSymbol *);

/* Fournit un étiquette pour viser un symbole. */
const char *g_binary_symbol_get_label(const GBinSymbol *);

/* Définit un autre nom pour le symbole. */
void g_binary_symbol_set_alt_label(GBinSymbol *, const char *);

/* Raffine la définition de l'emplacement d'un symbole. */
void g_binary_symbol_fix_range(GBinSymbol *, const vmpa2t *);

/* Fournit l'emplacement où se situe un symbole. */
const mrange_t *g_binary_symbol_get_range(const GBinSymbol *);

/* Attache la routine associée au symbole. */
void _g_binary_symbol_attach_routine(GBinSymbol *, GBinRoutine *, SymbolType);

/* Attache la routine associée au symbole. */
void g_binary_symbol_attach_routine(GBinSymbol *, GBinRoutine *);

/* Attache l'instruction associée au symbole. */
void g_binary_symbol_attach_instruction(GBinSymbol *, GArchInstruction *);

/* Fournit l'éventuelle routine associée au symbole. */
GBinRoutine *g_binary_symbol_get_routine(const GBinSymbol *);

/* Fournit l'éventuelle instruction associée au symbole. */
GArchInstruction *g_binary_symbol_get_instruction(const GBinSymbol *);

/* Ajoute un commentaire facultatif au symbole. */
void g_binary_symbol_set_comment(GBinSymbol *, GDbComment *);

/* Fournit l'éventuel commentaire associé au symbole. */
GDbComment *g_binary_symbol_get_comment(const GBinSymbol *);


/**
 * Confort pour l'ajout de symboles basés sur des formats.
 */

#define SET_IMM_DISPLAY(_ins, _op, _idx, _dsp)                          \
    do                                                                  \
    {                                                                   \
        _op = g_arch_instruction_get_operand(_ins, _idx);               \
        g_imm_operand_set_default_display(G_IMM_OPERAND(_op), _dsp);    \
        g_imm_operand_set_display(G_IMM_OPERAND(_op), _dsp);            \
    }                                                                   \
    while (0)

#define ADD_RAW_AS_SYM(_fmt, _sym, _ins, _cmt, _txt)                        \
    do                                                                      \
    {                                                                       \
        const vmpa2t *__addr;                                               \
        __addr = get_mrange_addr(g_arch_instruction_get_range(_ins));       \
        _cmt = g_db_comment_new_inlined(__addr, BLF_HAS_CODE, _txt, false); \
        g_db_item_set_volatile(G_DB_ITEM(_cmt), true);                      \
        _sym = g_binary_symbol_new(STP_DATA);                               \
        g_binary_symbol_attach_instruction(_sym, _ins);                     \
        g_binary_symbol_set_comment(_sym, _cmt);                            \
        g_binary_format_add_symbol(G_BIN_FORMAT(_fmt), _sym);               \
    }                                                                       \
    while (0)

#define ADD_STR_AS_SYM(_fmt, _sym, _ins)                        \
    do                                                          \
    {                                                           \
        _sym = g_binary_symbol_new(STP_RO_STRING);              \
        g_binary_symbol_attach_instruction(_sym, _ins);         \
        g_binary_format_add_symbol(G_BIN_FORMAT(_fmt), _sym);   \
    }                                                           \
    while (0)



/* ------------------------ OFFRE DE CAPACITES DE GENERATION ------------------------ */


/* Détermine si un symbole pour faire office de générateur. */
GLineGenerator *g_binary_symbol_produce_label(GBinSymbol *);



#endif  /* _FORMAT_SYMBOL_H */
